@@ -1,27 +1,31 @@
+from __future__ import division, absolute_import, print_function
+
 import re
 import os
 import sys
-import new
+import types
+from copy import copy
 
 from distutils.ccompiler import *
 from distutils import ccompiler
+from distutils.errors import DistutilsExecError, DistutilsModuleError, \
+                             DistutilsPlatformError
 from distutils.sysconfig import customize_compiler
 from distutils.version import LooseVersion
 
 from numpy.distutils import log
 from numpy.distutils.exec_command import exec_command
-from numpy.distutils.misc_util import cyg2win32, is_sequence, mingw32, quote_args, msvc_on_amd64
+from numpy.distutils.misc_util import cyg2win32, is_sequence, mingw32, \
+                                      quote_args
+from numpy.distutils.compat import get_exception
 
-# hack to set compiler optimizing options. Needs to integrated with something.
-import distutils.sysconfig
-_old_init_posix = distutils.sysconfig._init_posix
-def _new_init_posix():
-    _old_init_posix()
-    distutils.sysconfig._config_vars['OPT'] = '-Wall -g -O0'
-#distutils.sysconfig._init_posix = _new_init_posix
 
 def replace_method(klass, method_name, func):
-    m = new.instancemethod(func, None, klass)
+    if sys.version_info[0] < 3:
+        m = types.MethodType(func, None, klass)
+    else:
+        # Py3k does not have unbound method anymore, MethodType does not work
+        m = lambda self, *args, **kw: func(self, *args, **kw)
     setattr(klass, method_name, m)
 
 # Using customized CCompiler.spawn.
@@ -52,17 +56,20 @@ def CCompiler_spawn(self, cmd, display=None):
         if is_sequence(display):
             display = ' '.join(list(display))
     log.info(display)
-    s,o = exec_command(cmd)
+    s, o = exec_command(cmd)
     if s:
         if is_sequence(cmd):
             cmd = ' '.join(list(cmd))
-        print o
+        try:
+            print(o)
+        except UnicodeError:
+            # When installing through pip, `o` can contain non-ascii chars
+            pass
         if re.search('Too many open files', o):
             msg = '\nTry rerunning setup command until build succeeds.'
         else:
             msg = ''
-        raise DistutilsExecError,\
-              'Command "%s" failed with exit status %d%s' % (cmd, s, msg)
+        raise DistutilsExecError('Command "%s" failed with exit status %d%s' % (cmd, s, msg))
 
 replace_method(CCompiler, 'spawn', CCompiler_spawn)
 
@@ -104,11 +111,10 @@ def CCompiler_object_filenames(self, source_filenames, strip_dir=0, output_dir='
             d = os.path.basename(os.path.abspath(d))
             base = d + base[i:]
         if ext not in self.src_extensions:
-            raise UnknownFileError, \
-                  "unknown file type '%s' (from '%s')" % (ext, src_name)
+            raise UnknownFileError("unknown file type '%s' (from '%s')" % (ext, src_name))
         if strip_dir:
             base = os.path.basename(base)
-        obj_name = os.path.join(output_dir,base + self.obj_extension)
+        obj_name = os.path.join(output_dir, base + self.obj_extension)
         obj_names.append(obj_name)
     return obj_names
 
@@ -157,11 +163,15 @@ def CCompiler_compile(self, sources, output_dir=None, macros=None,
     # method to support pre Python 2.3 distutils.
     if not sources:
         return []
-    from fcompiler import FCompiler
+    # FIXME:RELATIVE_IMPORT
+    if sys.version_info[0] < 3:
+        from .fcompiler import FCompiler
+    else:
+        from numpy.distutils.fcompiler import FCompiler
     if isinstance(self, FCompiler):
         display = []
-        for fc in ['f77','f90','fix']:
-            fcomp = getattr(self,'compiler_'+fc)
+        for fc in ['f77', 'f90', 'fix']:
+            fcomp = getattr(self, 'compiler_'+fc)
             if fcomp is None:
                 continue
             display.append("Fortran %s compiler: %s" % (fc, ' '.join(fcomp)))
@@ -182,7 +192,7 @@ def CCompiler_compile(self, sources, output_dir=None, macros=None,
     # build any sources in same order as they were originally specified
     #   especially important for fortran .f90 files using modules
     if isinstance(self, FCompiler):
-        objects_to_build = build.keys()
+        objects_to_build = list(build.keys())
         for obj in objects:
             if obj in objects_to_build:
                 src, ext = build[obj]
@@ -226,7 +236,7 @@ def CCompiler_customize_cmd(self, cmd, ignore=()):
     if allow('include_dirs'):
         self.set_include_dirs(cmd.include_dirs)
     if allow('define'):
-        for (name,value) in cmd.define:
+        for (name, value) in cmd.define:
             self.define_macro(name, value)
     if allow('undef'):
         for macro in cmd.undef:
@@ -245,17 +255,17 @@ replace_method(CCompiler, 'customize_cmd', CCompiler_customize_cmd)
 def _compiler_to_string(compiler):
     props = []
     mx = 0
-    keys = compiler.executables.keys()
-    for key in ['version','libraries','library_dirs',
-                'object_switch','compile_switch',
-                'include_dirs','define','undef','rpath','link_objects']:
+    keys = list(compiler.executables.keys())
+    for key in ['version', 'libraries', 'library_dirs',
+                'object_switch', 'compile_switch',
+                'include_dirs', 'define', 'undef', 'rpath', 'link_objects']:
         if key not in keys:
             keys.append(key)
     for key in keys:
-        if hasattr(compiler,key):
+        if hasattr(compiler, key):
             v = getattr(compiler, key)
-            mx = max(mx,len(key))
-            props.append((key,repr(v)))
+            mx = max(mx, len(key))
+            props.append((key, repr(v)))
     lines = []
     format = '%-' + repr(mx+1) + 's = %s'
     for prop in props:
@@ -280,22 +290,22 @@ def CCompiler_show_customization(self):
 
     """
     if 0:
-        for attrname in ['include_dirs','define','undef',
-                         'libraries','library_dirs',
-                         'rpath','link_objects']:
-            attr = getattr(self,attrname,None)
+        for attrname in ['include_dirs', 'define', 'undef',
+                         'libraries', 'library_dirs',
+                         'rpath', 'link_objects']:
+            attr = getattr(self, attrname, None)
             if not attr:
                 continue
-            log.info("compiler '%s' is set to %s" % (attrname,attr))
+            log.info("compiler '%s' is set to %s" % (attrname, attr))
     try:
         self.get_version()
     except:
         pass
     if log._global_log.threshold<2:
-        print '*'*80
-        print self.__class__
-        print _compiler_to_string(self)
-        print '*'*80
+        print('*'*80)
+        print(self.__class__)
+        print(_compiler_to_string(self))
+        print('*'*80)
 
 replace_method(CCompiler, 'show_customization', CCompiler_show_customization)
 
@@ -335,27 +345,22 @@ def CCompiler_customize(self, dist, need_cxx=0):
     if need_cxx:
         # In general, distutils uses -Wstrict-prototypes, but this option is
         # not valid for C++ code, only for C.  Remove it if it's there to
-        # avoid a spurious warning on every compilation.  All the default
-        # options used by distutils can be extracted with:
-
-        # from distutils import sysconfig
-        # sysconfig.get_config_vars('CC', 'CXX', 'OPT', 'BASECFLAGS',
-        # 'CCSHARED', 'LDSHARED', 'SO')
+        # avoid a spurious warning on every compilation.
         try:
             self.compiler_so.remove('-Wstrict-prototypes')
         except (AttributeError, ValueError):
             pass
 
-        if hasattr(self,'compiler') and 'cc' in self.compiler[0]:
+        if hasattr(self, 'compiler') and 'cc' in self.compiler[0]:
             if not self.compiler_cxx:
                 if self.compiler[0].startswith('gcc'):
                     a, b = 'gcc', 'g++'
                 else:
                     a, b = 'cc', 'c++'
-                self.compiler_cxx = [self.compiler[0].replace(a,b)]\
+                self.compiler_cxx = [self.compiler[0].replace(a, b)]\
                                     + self.compiler[1:]
         else:
-            if hasattr(self,'compiler'):
+            if hasattr(self, 'compiler'):
                 log.warn("#### %s #######" % (self.compiler,))
             log.warn('Missing compiler_cxx fix for '+self.__class__.__name__)
     return
@@ -389,13 +394,16 @@ def simple_version_match(pat=r'[-.\d]+', ignore='', start=''):
 
     """
     def matcher(self, version_string):
+        # version string may appear in the second line, so getting rid
+        # of new lines:
+        version_string = version_string.replace('\n', ' ')
         pos = 0
         if start:
             m = re.match(start, version_string)
             if not m:
                 return None
             pos = m.end()
-        while 1:
+        while True:
             m = re.search(pat, version_string[pos:])
             if not m:
                 return None
@@ -426,7 +434,7 @@ def CCompiler_get_version(self, force=False, ok_status=[0]):
         Version string, in the format of `distutils.version.LooseVersion`.
 
     """
-    if not force and hasattr(self,'version'):
+    if not force and hasattr(self, 'version'):
         return self.version
     self.find_executables()
     try:
@@ -449,7 +457,7 @@ def CCompiler_get_version(self, force=False, ok_status=[0]):
             version = m.group('version')
             return version
 
-    status, output = exec_command(version_cmd,use_tee=0)
+    status, output = exec_command(version_cmd, use_tee=0)
 
     version = None
     if status in ok_status:
@@ -488,11 +496,18 @@ def CCompiler_cxx_compiler(self):
 
 replace_method(CCompiler, 'cxx_compiler', CCompiler_cxx_compiler)
 
-compiler_class['intel'] = ('intelccompiler','IntelCCompiler',
+compiler_class['intel'] = ('intelccompiler', 'IntelCCompiler',
                            "Intel C Compiler for 32-bit applications")
-compiler_class['intele'] = ('intelccompiler','IntelItaniumCCompiler',
+compiler_class['intele'] = ('intelccompiler', 'IntelItaniumCCompiler',
                            "Intel C Itanium Compiler for Itanium-based applications")
-ccompiler._default_compilers += (('linux.*','intel'),('linux.*','intele'))
+compiler_class['intelem'] = ('intelccompiler', 'IntelEM64TCCompiler',
+                             "Intel C Compiler for 64-bit applications")
+compiler_class['pathcc'] = ('pathccompiler', 'PathScaleCCompiler',
+                            "PathScale Compiler for SiCortex-based applications")
+ccompiler._default_compilers += (('linux.*', 'intel'),
+                                 ('linux.*', 'intele'),
+                                 ('linux.*', 'intelem'),
+                                 ('linux.*', 'pathcc'))
 
 if sys.platform == 'win32':
     compiler_class['mingw32'] = ('mingw32ccompiler', 'Mingw32CCompiler',
@@ -523,27 +538,27 @@ def new_compiler (plat=None,
         msg = "don't know how to compile C/C++ code on platform '%s'" % plat
         if compiler is not None:
             msg = msg + " with '%s' compiler" % compiler
-        raise DistutilsPlatformError, msg
+        raise DistutilsPlatformError(msg)
     module_name = "numpy.distutils." + module_name
     try:
         __import__ (module_name)
-    except ImportError, msg:
+    except ImportError:
+        msg = str(get_exception())
         log.info('%s in numpy.distutils; trying from distutils',
                  str(msg))
         module_name = module_name[6:]
         try:
             __import__(module_name)
-        except ImportError, msg:
-            raise DistutilsModuleError, \
-                  "can't compile C/C++ code: unable to load module '%s'" % \
-                  module_name
+        except ImportError:
+            msg = str(get_exception())
+            raise DistutilsModuleError("can't compile C/C++ code: unable to load module '%s'" % \
+                  module_name)
     try:
         module = sys.modules[module_name]
         klass = vars(module)[class_name]
     except KeyError:
-        raise DistutilsModuleError, \
-              ("can't compile C/C++ code: unable to find class '%s' " +
-               "in module '%s'") % (class_name, module_name)
+        raise DistutilsModuleError(("can't compile C/C++ code: unable to find class '%s' " +
+               "in module '%s'") % (class_name, module_name))
     compiler = klass(None, dry_run, force)
     log.debug('new_compiler returns %s' % (klass))
     return compiler
@@ -568,8 +583,8 @@ ccompiler.gen_lib_options = gen_lib_options
 # Also fix up the various compiler modules, which do
 # from distutils.ccompiler import gen_lib_options
 # Don't bother with mwerks, as we don't support Classic Mac.
-for _cc in ['msvc', 'bcpp', 'cygwinc', 'emxc', 'unixc']:
-    _m = sys.modules.get('distutils.'+_cc+'compiler')
+for _cc in ['msvc9', 'msvc', 'bcpp', 'cygwinc', 'emxc', 'unixc']:
+    _m = sys.modules.get('distutils.' + _cc + 'compiler')
     if _m is not None:
         setattr(_m, 'gen_lib_options', gen_lib_options)
 
@@ -591,7 +606,7 @@ _squote_re = re.compile(r"'(?:[^'\\]|\\.)*'")
 _dquote_re = re.compile(r'"(?:[^"\\]|\\.)*"')
 _has_white_re = re.compile(r'\s')
 def split_quoted(s):
-    s = string.strip(s)
+    s = s.strip()
     words = []
     pos = 0
 
@@ -604,7 +619,7 @@ def split_quoted(s):
 
         if s[end] in string.whitespace: # unescaped, unquoted whitespace: now
             words.append(s[:end])       # we definitely have a word delimiter
-            s = string.lstrip(s[end:])
+            s = s[end:].lstrip()
             pos = 0
 
         elif s[end] == '\\':            # preserve whatever is being escaped;
@@ -618,12 +633,10 @@ def split_quoted(s):
             elif s[end] == '"':         # slurp doubly-quoted string
                 m = _dquote_re.match(s, end)
             else:
-                raise RuntimeError, \
-                      "this can't happen (bad char '%c')" % s[end]
+                raise RuntimeError("this can't happen (bad char '%c')" % s[end])
 
             if m is None:
-                raise ValueError, \
-                      "bad string (mismatched %s quotes?)" % s[end]
+                raise ValueError("bad string (mismatched %s quotes?)" % s[end])
 
             (beg, end) = m.span()
             if _has_white_re.search(s[beg+1:end-1]):
@@ -641,6 +654,3 @@ def split_quoted(s):
     return words
 ccompiler.split_quoted = split_quoted
 ##Fix distutils.util.split_quoted:
-
-# define DISTUTILS_USE_SDK when necessary to workaround distutils/msvccompiler.py bug
-msvc_on_amd64()
