@@ -1,5 +1,7 @@
 """ Modified version of build_ext that handles fortran source files.
+
 """
+from __future__ import division, absolute_import, print_function
 
 import os
 import sys
@@ -35,7 +37,7 @@ class build_ext (old_build_ext):
         ]
 
     help_options = old_build_ext.help_options + [
-        ('help-fcompiler',None, "list available Fortran compilers",
+        ('help-fcompiler', None, "list available Fortran compilers",
          show_fortran_compilers),
         ]
 
@@ -44,10 +46,22 @@ class build_ext (old_build_ext):
         self.fcompiler = None
 
     def finalize_options(self):
-        incl_dirs = self.include_dirs
+        # Ensure that self.include_dirs and self.distribution.include_dirs
+        # refer to the same list object. finalize_options will modify
+        # self.include_dirs, but self.distribution.include_dirs is used
+        # during the actual build.
+        # self.include_dirs is None unless paths are specified with
+        # --include-dirs.
+        # The include paths will be passed to the compiler in the order:
+        # numpy paths, --include-dirs paths, Python include path.
+        if isinstance(self.include_dirs, str):
+            self.include_dirs = self.include_dirs.split(os.pathsep)
+        incl_dirs = self.include_dirs or []
+        if self.distribution.include_dirs is None:
+            self.distribution.include_dirs = []
+        self.include_dirs = self.distribution.include_dirs
+        self.include_dirs.extend(incl_dirs)
         old_build_ext.finalize_options(self)
-        if incl_dirs is not None:
-            self.include_dirs.extend(self.distribution.include_dirs or [])
 
     def run(self):
         if not self.extensions:
@@ -57,8 +71,21 @@ class build_ext (old_build_ext):
         self.run_command('build_src')
 
         if self.distribution.has_c_libraries():
-            self.run_command('build_clib')
-            build_clib = self.get_finalized_command('build_clib')
+            if self.inplace:
+                if self.distribution.have_run.get('build_clib'):
+                    log.warn('build_clib already run, it is too late to ' \
+                            'ensure in-place build of build_clib')
+                    build_clib = self.distribution.get_command_obj('build_clib')
+                else:
+                    build_clib = self.distribution.get_command_obj('build_clib')
+                    build_clib.inplace = 1
+                    build_clib.ensure_finalized()
+                    build_clib.run()
+                    self.distribution.have_run['build_clib'] = 1
+
+            else:
+                self.run_command('build_clib')
+                build_clib = self.get_finalized_command('build_clib')
             self.library_dirs.append(build_clib.build_clib)
         else:
             build_clib = None
@@ -84,14 +111,14 @@ class build_ext (old_build_ext):
         # Create mapping of libraries built by build_clib:
         clibs = {}
         if build_clib is not None:
-            for libname,build_info in build_clib.libraries or []:
+            for libname, build_info in build_clib.libraries or []:
                 if libname in clibs and clibs[libname] != build_info:
                     log.warn('library %r defined more than once,'\
                              ' overwriting build_info\n%s... \nwith\n%s...' \
-                             % (libname, `clibs[libname]`[:300], `build_info`[:300]))
+                             % (libname, repr(clibs[libname])[:300], repr(build_info)[:300]))
                 clibs[libname] = build_info
         # .. and distribution libraries:
-        for libname,build_info in self.distribution.libraries or []:
+        for libname, build_info in self.distribution.libraries or []:
             if libname in clibs:
                 # build_clib libraries have a precedence before distribution ones
                 continue
@@ -108,13 +135,13 @@ class build_ext (old_build_ext):
             for libname in ext.libraries:
                 if libname in clibs:
                     binfo = clibs[libname]
-                    c_libs += binfo.get('libraries',[])
-                    c_lib_dirs += binfo.get('library_dirs',[])
-                    for m in binfo.get('macros',[]):
+                    c_libs += binfo.get('libraries', [])
+                    c_lib_dirs += binfo.get('library_dirs', [])
+                    for m in binfo.get('macros', []):
                         if m not in macros:
                             macros.append(m)
 
-                for l in clibs.get(libname,{}).get('source_languages',[]):
+                for l in clibs.get(libname, {}).get('source_languages', []):
                     ext_languages.add(l)
             if c_libs:
                 new_c_libs = ext.libraries + c_libs
@@ -146,7 +173,7 @@ class build_ext (old_build_ext):
                 ext_language = 'c' # default
             if l and l != ext_language and ext.language:
                 log.warn('resetting extension %r language from %r to %r.' %
-                         (ext.name,l,ext_language))
+                         (ext.name, l, ext_language))
             ext.language = ext_language
             # global language
             all_languages.update(ext_languages)
@@ -162,7 +189,7 @@ class build_ext (old_build_ext):
                                              dry_run=self.dry_run,
                                              force=self.force)
             compiler = self._cxx_compiler
-            compiler.customize(self.distribution,need_cxx=need_cxx_compiler)
+            compiler.customize(self.distribution, need_cxx=need_cxx_compiler)
             compiler.customize_cmd(self)
             compiler.show_customization()
             self._cxx_compiler = compiler.cxx_compiler()
@@ -218,11 +245,6 @@ class build_ext (old_build_ext):
         # Build extensions
         self.build_extensions()
 
-        # Make sure that scons based extensions are complete.
-        if self.inplace:
-            cmd = self.reinitialize_command('scons')
-            cmd.inplace = 1
-        self.run_command('scons')
 
     def swig_sources(self, sources):
         # Do nothing. Swig sources have beed handled in build_src command.
@@ -286,16 +308,19 @@ class build_ext (old_build_ext):
             fcompiler = self._f77_compiler
         else: # in case ext.language is c++, for instance
             fcompiler = self._f90_compiler or self._f77_compiler
+        if fcompiler is not None:
+            fcompiler.extra_f77_compile_args = (ext.extra_f77_compile_args or []) if hasattr(ext, 'extra_f77_compile_args') else []
+            fcompiler.extra_f90_compile_args = (ext.extra_f90_compile_args or []) if hasattr(ext, 'extra_f90_compile_args') else []
         cxx_compiler = self._cxx_compiler
 
         # check for the availability of required compilers
         if cxx_sources and cxx_compiler is None:
-            raise DistutilsError, "extension %r has C++ sources" \
-                  "but no C++ compiler found" % (ext.name)
+            raise DistutilsError("extension %r has C++ sources" \
+                  "but no C++ compiler found" % (ext.name))
         if (f_sources or fmodule_sources) and fcompiler is None:
-            raise DistutilsError, "extension %r has Fortran sources " \
-                  "but no Fortran compiler found" % (ext.name)
-        if ext.language in ['f77','f90'] and fcompiler is None:
+            raise DistutilsError("extension %r has Fortran sources " \
+                  "but no Fortran compiler found" % (ext.name))
+        if ext.language in ['f77', 'f90'] and fcompiler is None:
             self.warn("extension %r has Fortran libraries " \
                   "but no Fortran linker found, using default linker" % (ext.name))
         if ext.language=='c++' and cxx_compiler is None:
@@ -334,14 +359,14 @@ class build_ext (old_build_ext):
             log.info("compiling Fortran 90 module sources")
             module_dirs = ext.module_dirs[:]
             module_build_dir = os.path.join(
-                self.build_temp,os.path.dirname(
+                self.build_temp, os.path.dirname(
                     self.get_ext_filename(fullname)))
 
             self.mkpath(module_build_dir)
             if fcompiler.module_dir_switch is None:
                 existing_modules = glob('*.mod')
             extra_postargs += fcompiler.module_options(
-                module_dirs,module_build_dir)
+                module_dirs, module_build_dir)
             f_objects += fcompiler.compile(fmodule_sources,
                                            output_dir=self.build_temp,
                                            macros=macros,
@@ -389,7 +414,7 @@ class build_ext (old_build_ext):
             # not using fcompiler linker
             self._libs_with_msvc_and_fortran(fcompiler, libraries, library_dirs)
 
-        elif ext.language in ['f77','f90'] and fcompiler is not None:
+        elif ext.language in ['f77', 'f90'] and fcompiler is not None:
             linker = fcompiler.link_shared_object
         if ext.language=='c++' and cxx_compiler is not None:
             linker = cxx_compiler.link_shared_object
@@ -409,10 +434,10 @@ class build_ext (old_build_ext):
                build_temp=self.build_temp,**kws)
 
     def _add_dummy_mingwex_sym(self, c_sources):
-	build_src = self.get_finalized_command("build_src").build_src
-	build_clib = self.get_finalized_command("build_clib").build_clib
-	objects = self.compiler.compile([os.path.join(build_src,
-		"gfortran_vs2003_hack.c")],
+        build_src = self.get_finalized_command("build_src").build_src
+        build_clib = self.get_finalized_command("build_clib").build_clib
+        objects = self.compiler.compile([os.path.join(build_src,
+                "gfortran_vs2003_hack.c")],
                 output_dir=self.build_temp)
         self.compiler.create_static_lib(objects, "_gfortran_workaround", output_dir=build_clib, debug=self.debug)
 
@@ -424,7 +449,7 @@ class build_ext (old_build_ext):
             if libname.startswith('msvc'): continue
             fileexists = False
             for libdir in c_library_dirs or []:
-                libfile = os.path.join(libdir,'%s.lib' % (libname))
+                libfile = os.path.join(libdir, '%s.lib' % (libname))
                 if os.path.isfile(libfile):
                     fileexists = True
                     break
@@ -432,7 +457,7 @@ class build_ext (old_build_ext):
             # make g77-compiled static libs available to MSVC
             fileexists = False
             for libdir in c_library_dirs:
-                libfile = os.path.join(libdir,'lib%s.a' % (libname))
+                libfile = os.path.join(libdir, 'lib%s.a' % (libname))
                 if os.path.isfile(libfile):
                     # copy libname.a file to name.lib so that MSVC linker
                     # can find it
@@ -452,7 +477,7 @@ class build_ext (old_build_ext):
             # correct path when compiling in Cygwin but with normal Win
             # Python
             if dir.startswith('/usr/lib'):
-                s,o = exec_command(['cygpath', '-w', dir], use_tee=False)
+                s, o = exec_command(['cygpath', '-w', dir], use_tee=False)
                 if not s:
                     dir = o
             f_lib_dirs.append(dir)
